#include "HTableView.h"
#include <QtGui/QHeaderView>
#include <QtGui/QScrollBar>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtGui/QX11Info>

#if QT_VERSION >= 0x040200

// Foil busted word-wrapping that happens with decorationPosition = Bottom
// in Qt 4.2.

#include <QtGui/QItemDelegate>
#include <QtGui/QStyleOptionViewItemV2>

class HItemDelegate : public QItemDelegate
{
public: 
   HItemDelegate (QObject *parent = NULL)
      : QItemDelegate(parent) 
   {
   }
   virtual void drawDisplay(QPainter*                    painter,
                            const QStyleOptionViewItem&  option,
                            const QRect&                 rect,
                            const QString&               text) const
   {
      const QStyleOptionViewItemV2*  v2
         = qstyleoption_cast<const QStyleOptionViewItemV2*>(&option);
      if (v2) {
         QStyleOptionViewItemV2  v = *v2;
         v.features &= ~QStyleOptionViewItemV2::WrapText;
         QItemDelegate::drawDisplay(painter, v, rect, text); 
      } else {
         QItemDelegate::drawDisplay(painter, option, rect, text);
      }
   }
};

#endif /* QT_VERSION >= 0x040200 */

HTableView::HTableView(QStyleOptionViewItem::Position  pos,
                       int                             minRows,
                       int                             maxRows,
                       QWidget*                        parent)
   : QTableView(parent), pos(pos), minRows(minRows), maxRows(maxRows)
{
#if QT_VERSION >= 0x040200
   setItemDelegate(new HItemDelegate(parent));
#endif
}

QStyleOptionViewItem HTableView::viewOptions() const 
{
   QStyleOptionViewItem  opt = QTableView::viewOptions();
   opt.decorationPosition = pos;
   opt.showDecorationSelected = false;
   return opt;
}

int HTableView::frameOnlyAroundContentsPixels() const
{
   if (style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents, 
                          0, this)) {
      return style()->pixelMetric(QStyle::PM_DefaultFrameWidth) * 2;
   } else {
      return 0;
   }
}

QSize HTableView::sizeHint() const
{
   if (minRows != maxRows) {
      minRows = qMax(minRows, model()->rowCount());
      maxRows = minRows = qMin(minRows, maxRows);
   }

   // NOTE: The height returned from here is reduced by 23-28 pixels when
   //       the widget is first drawn.  I don't know why, but I suspect
   //       it believes the sizeHint is including the horizontal scrollbar
   //       and then is subtracting that out when it decides that it doesn't 
   //       need it.  I used to add 28 pixels (hard-coded) to workaround that.
   //       But it isn't needed any more in this tool because the widget is 
   //       resized after the application starts by resizeColumnsToContents 
   //       below.  And the sizeHint is honored when that happens.  Heaven
   //       knows why it's honored late but not early.

   int  lastColumn = model()->columnCount() - 1;
   int  w          = (frameWidth()
                      + (verticalHeader()->isHidden()
                         ? 0 : verticalHeader()->sizeHint().width())
                      + columnViewportPosition(lastColumn)
                      + columnWidth(lastColumn)
                      + frameOnlyAroundContentsPixels()
                      + verticalScrollBar()->sizeHint().width()
                      + frameWidth());
   int  h          = (frameWidth()
                      + (horizontalHeader()->isHidden()
                         ? 0 : horizontalHeader()->sizeHint().height())
                      + rowViewportPosition(0)
                      + (rowHeight(0) * minRows)
                      + frameWidth());

   return QSize(w, h);
}

void HTableView::sortByColumn(int column)
{
   if (model() == NULL) return;

   Qt::SortOrder  order;
   if (horizontalHeader()->sortIndicatorSection() == column) {
      order = horizontalHeader()->sortIndicatorOrder();
#if QT_VERSION < 0x040500
      // In Qt 4.4.x and prior, QHeaderView::mouseReleaseEvent does not
      // flip the sort indicator if it isn't shown.  So do that manually here.
      // In Qt 4.5.x and later, it does flip it, so just use it as is.
      if (!horizontalHeader()->isSortIndicatorShown()) {
         if (order == Qt::AscendingOrder) {
            order = Qt::DescendingOrder;
         } else {
            order = Qt::AscendingOrder;
         }
      }
#endif
   } else {
      // If changing the sort to a different column than a previous one, always
      // set it to an Ascending sort order, insteading of using whatever random
      // flip or no-flip happened while clicking on the header.
      order = Qt::AscendingOrder;
   }
   horizontalHeader()->setSortIndicator(column, order);
   model()->sort(column, order);
}

void  HTableView::sortByColumn(int column, Qt::SortOrder order)
{
   // QTableView::sortByColumn likes to call the 1-argument form.  But we have
   // some kludge magic in our version.  So just make the calls directly to
   // avoid all that.
   horizontalHeader()->setSortIndicator(column, order);
   model()->sort(column, order);
}

void  HTableView::resizeColumnsToContents()
{
   QTableView::resizeColumnsToContents();

   QWidget*  widget = this;
   for (; widget != NULL; widget = widget->parentWidget()) {
      widget->updateGeometry();
   }

   QRect  screen  = QApplication::desktop()->screenGeometry(x11Info().screen());
   QSize  newSize = window()->sizeHint().boundedTo(QSize(screen.width()*9/10,
                                                         screen.height()*9/10))
                                        .expandedTo(window()->size());

   window()->resize(newSize);
}

int HTableView::sizeHintForColumn(int column) const
{
   if (model() == NULL) return -1;

   int                   result = 0;
   QStyleOptionViewItem  option = viewOptions();
   int                   tail   = model()->rowCount(rootIndex());
   int                   row;

   for (row = 0; row < tail; ++row) {
      QModelIndex  index = model()->index(row, column, rootIndex());
      result = qMax(result, itemDelegate()->sizeHint(option, index).width());
   }

   if (showGrid()) {
      result++;
   }

   return result;
}

QModelIndexList  HTableView::selectedRows() const
{
   QItemSelection                  sel    = selectionModel()->selection();
   QModelIndexList                 result;
   QItemSelection::const_iterator  it;

   for (it = sel.begin(); it != sel.end(); ++it) {
      int  left = it->left();
      int  row;
      for (row = it->top(); row <= it->bottom(); row++) {
         QModelIndex  ix = model()->index(row, left, it->parent());
         if (it->model()->flags(ix) & Qt::ItemIsSelectable) {
            result.append(ix);
         }
      }
   }

   return result;
}
